@@ -1,12 +1,11 @@
 import m3u8
-import os
-import pathlib
 import re
 import requests
 import shutil
 import subprocess
 import tempfile
 
+from os import path
 from pathlib import Path
 from urllib.parse import urlparse
 
@@ -103,24 +102,20 @@ def _select_playlist_interactive(playlists):
     return uri
 
 
-def _join_vods(directory, file_paths, target):
-    input_path = "{}/files.txt".format(directory)
-
-    with open(input_path, 'w') as f:
-        for path in file_paths:
-            f.write('file {}\n'.format(os.path.basename(path)))
-
-    result = subprocess.run([
+def _join_vods(playlist_path, target):
+    command = [
         "ffmpeg",
-        "-f", "concat",
-        "-i", input_path,
+        "-i", playlist_path,
         "-c", "copy",
         target,
         "-stats",
         "-loglevel", "warning",
-    ])
+    ]
 
-    result.check_returncode()
+    print_out("<dim>{}</dim>".format(" ".join(command)))
+    result = subprocess.run(command)
+    if result.returncode != 0:
+        raise ConsoleError("Joining files failed")
 
 
 def _video_target_filename(video, format):
@@ -137,8 +132,9 @@ def _video_target_filename(video, format):
     return name + "." + format
 
 
-def _get_files(playlist, start, end):
-    """Extract files for download from playlist."""
+def _get_vod_paths(playlist, start, end):
+    """Extract unique VOD paths for download from playlist."""
+    files = []
     vod_start = 0
     for segment in playlist.segments:
         vod_end = vod_start + segment.duration
@@ -148,18 +144,20 @@ def _get_files(playlist, start, end):
         start_condition = not start or vod_end > start
         end_condition = not end or vod_start < end
 
-        if start_condition and end_condition:
-            yield segment.uri
+        if start_condition and end_condition and segment.uri not in files:
+            files.append(segment.uri)
 
         vod_start = vod_end
 
+    return files
+
 
 def _crete_temp_dir(base_uri):
     """Create a temp dir to store downloads if it doesn't exist."""
-    path = urlparse(base_uri).path
-    directory = '{}/twitch-dl{}'.format(tempfile.gettempdir(), path)
-    pathlib.Path(directory).mkdir(parents=True, exist_ok=True)
-    return directory
+    path = urlparse(base_uri).path.lstrip("/")
+    temp_dir = Path(tempfile.gettempdir(), "twitch-dl", path)
+    temp_dir.mkdir(parents=True, exist_ok=True)
+    return temp_dir
 
 
 VIDEO_PATTERNS = [
@@ -272,26 +270,38 @@ def _download_video(video_id, args):
 
     base_uri = re.sub("/[^/]+$", "/", playlist_uri)
     target_dir = _crete_temp_dir(base_uri)
-    filenames = list(_get_files(playlist, args.start, args.end))
+    vod_paths = _get_vod_paths(playlist, args.start, args.end)
 
     # Save playlists for debugging purposes
-    with open(target_dir + "playlists.m3u8", "w") as f:
+    with open(path.join(target_dir, "playlists.m3u8"), "w") as f:
         f.write(playlists_m3u8)
-    with open(target_dir + "playlist.m3u8", "w") as f:
+    with open(path.join(target_dir, "playlist.m3u8"), "w") as f:
         f.write(response.text)
 
     print_out("\nDownloading {} VODs using {} workers to {}".format(
-        len(filenames), args.max_workers, target_dir))
-    file_paths = download_files(base_uri, target_dir, filenames, args.max_workers)
+        len(vod_paths), args.max_workers, target_dir))
+    path_map = download_files(base_uri, target_dir, vod_paths, args.max_workers)
+
+    # Make a modified playlist which references downloaded VODs
+    # Keep only the downloaded segments and skip the rest
+    org_segments = playlist.segments.copy()
+    playlist.segments.clear()
+    for segment in org_segments:
+        if segment.uri in path_map:
+            segment.uri = path_map[segment.uri]
+            playlist.segments.append(segment)
+
+    playlist_path = path.join(target_dir, "playlist_downloaded.m3u8")
+    playlist.dump(playlist_path)
 
     print_out("\n\nJoining files...")
     target = _video_target_filename(video, args.format)
-    _join_vods(target_dir, file_paths, target)
+    _join_vods(playlist_path, target)
 
     if args.keep:
-        print_out("\nTemporary files not deleted: {}".format(target_dir))
+        print_out("\n<dim>Temporary files not deleted: {}</dim>".format(target_dir))
     else:
-        print_out("\nDeleting temporary files...")
+        print_out("\n<dim>Deleting temporary files...</dim>")
         shutil.rmtree(target_dir)
 
-    print_out("Downloaded: {}".format(target))
+    print_out("\nDownloaded: <green>{}</green>".format(target))
